/*
 * Copyright 2002 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 */

#ifndef _DAG_H_
#define _DAG_H_
#include <sys/tree.h>

struct dag_edge;

struct dag_point {
	double x;
	double y;
};

#define p_x point.x
#define p_y point.y

struct dag_node {
	TAILQ_ENTRY (dag_node) next;
	TAILQ_ENTRY (dag_node) bucket_next;
	SPLAY_ENTRY (dag_node) node;
	SPLAY_ENTRY (dag_node) id_node;

	SPLAY_HEAD (edgetree, dag_edge) edgeroot;

	int id;

	struct dag_point point;
	struct dag_point force;
	double energy;

	struct dag_point save_point;
	struct dag_point save_force;
	double save_energy;

	struct dag_point old_force;
	double old_energy;

	double size;
};

struct dag_edge {
	SPLAY_ENTRY (dag_edge) node;

	struct dag_node *destination;
	struct dag_edge *undirected;
};

void dag_init(void);

struct dag_node *dag_new_node(int, int);
struct dag_node *dag_node_findid(int);
void dag_insert_node(struct dag_node *);
void dag_remove_node(struct dag_node *);
void dag_free_node(struct dag_node *);
int dag_walk_all(int (*cb)(struct dag_node *, void *), void *arg);
struct dag_node *dag_random_node(void);

void dag_move_node(struct dag_node *, struct dag_point *);

void dag_hash(void);
int dag_walk_region(int (*cb)(struct dag_node *, void *), int, int, int, int, void *);

void dag_connect(struct dag_node *, struct dag_node *, int);
void dag_deconnect(struct dag_node *, struct dag_node *);
int dag_isconnected(struct dag_node *, struct dag_node *);

double dag_distance(struct dag_node *, struct dag_node *);
struct dag_point *dag_repulse(struct dag_node *, struct dag_node *, double);
double dag_compute_forces(void);
void dag_apply_forces(void);
void dag_find_minimum(void);

void dag_update_max(int, int);
void dag_set_max(int, int);

void dag_set_params(double, double, double, double);
void dag_get_params(double *, double *, double *, double *);
void dag_get_size(int *, int *);

void dag_set_physics(double, double);
void dag_get_physics(double *, double *);

double dag_set_stepsize(double);

int dag_load_topology(char *name);
int dag_save_topology(char *name);

int edgecompare(struct dag_edge *, struct dag_edge *);
SPLAY_PROTOTYPE(edgetree, dag_edge, node, edgecompare);

#endif
